/*
	Previewer cum Scene Modeler using of Radiance Input format.
	USAGE : <program> [options] <Radiance Input File>
	Options : 
		-b bouding cube extents
		-s window_x_size window_y_size.
		-p window_x_posn window_y_posn.
		-v ONE/FOUR
		-c n ........For Circle Drawing Precision.
------
sumant (sumant@ncst.ernet.in)
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "preview.h"

FILE	*fp;
char	*program_name;
char	filename[80];

/*
 * Default Environment Values.
 */
int	circle_precision = 12; /* Circle Drawing Precision */
int	window_x_size=640, window_y_size=640;
int	window_x_posn=0, window_y_posn=0;
int	view_type = ONE;
float 	xmin=-1.,ymin=-1.,zmin=-1.,xmax=1.,ymax=1.,zmax=1.;

usage()
{
	fprintf(stderr, "usage: %s [-options] Radfile\n",program_name);
	fprintf(stderr,"options :\n");
	fprintf(stderr,"\t-b xmin xmax ymin ymax zmin zmax.\n"); 
	fprintf(stderr,"\t\tDefault bounding cube extents are :\n");
	fprintf(stderr,"\t\txmin:%g xmax:%g ymin:%g ymax:%g zmin:%g zmax:%g\n",
			xmin,xmax,ymin,ymax,zmin,zmax);
	fprintf(stderr,"\t-c circle_precision. Default %d.\n",circle_precision);
	fprintf(stderr,"\t-p window_x_posn window_y_posn. Default %d %d.\n",
		window_x_posn,window_y_posn);
	fprintf(stderr,"\t-s window_x_size window_y_size. Default %d %d.\n",
		window_x_size,window_y_size);
	fprintf(stderr,"\t-v ONE/FOUR. View type Default %s.\n",
			(view_type==ONE)?"ONE":"FOUR");
	exit(1);
}

parse(argc, argv)
         int     argc;
         char    **argv;
{
	int	c=1;

	program_name = argv[0];
	
	while((c < argc)&&(argv[c][0]=='-')){
		int ch = argv[c][1]; c++;
		switch (ch){
		case 'b':
		case 'B':
			if (c<argc){
				xmin=atof(argv[c++]);
				if (c<argc){
					xmax=atof(argv[c++]);
					if (c<argc){
						ymin=atof(argv[c++]);
						if (c<argc){
							ymax=atof(argv[c++]);
							if (c<argc){
								zmin=atof(argv[c++]);
								if (c<argc)
									zmax=atof(argv[c++]);
								else usage();
							}else usage();
						}else usage();
					}else usage();
				}else usage();
			}else usage();
			fprintf(stderr,"left : %g right : %g bottom : %g top : %g front : %g back : %g\n"
				,xmin,xmax,ymin,ymax,zmin,zmax);
			break;
		case 'c':
		case 'C': /* Circle Precision */
			if (c < argc){
				circle_precision = atoi(argv[c++]); 
				fprintf(stderr,
					"Circle is Displayed as %d sided Polygon.\n",
					circle_precision);
			}
			else usage();
			break;
		case 'p':
		case 'P': /* Window Position */
			if (c < argc){
				window_x_posn = atoi(argv[c++]);
				if (c < argc)
					window_y_posn = atoi(argv[c++]);
				else usage();
			}
			else usage();
			fprintf(stderr,
				"window x posn : %d, window y posn : %d.\n",
				window_x_posn,window_y_posn);
			break;
		case 's':
		case 'S': /* Window Size */
			if (c < argc){
				window_x_size = atoi(argv[c++]); 
				if (c < argc) window_y_size = atoi(argv[c++]);
				else usage();
			}
			else usage();
			fprintf(stderr,
				"window width : %d, window height : %d.\n",
				window_x_size,window_y_size);
			break;
		case 'v':
		case 'V': /* View Type */
			if (c < argc){
				int length = strlen(argv[c]);
				int i;
				for(i=0;i<length;i++)argv[c][i]=toupper(argv[c][i]);
				if (strcmp(argv[c],"ONE")==0) view_type=ONE;
				else if (strcmp(argv[c],"FOUR")==0)view_type=FOUR;
				else usage();
				c++;
			}
			else usage();
			fprintf(stderr,
				"%d View%s.\n",view_type,(view_type == FOUR)?"s":"");
			break;
		default : fprintf(stderr,"Wrong Option %s.");
			break;
		}
	}
	if (c < argc) strcpy(filename,argv[c++]);
	else usage();
}
help()
{
	fprintf(stderr,"Commands Available are :\n");
	fprintf(stderr,"\tr...For refreshing the screen.\n");
	fprintf(stderr,"\tn <filename>...To change the current file to a new file.\n");
	fprintf(stderr,"\t\tThe current file is %s.\n",filename);
	fprintf(stderr,"\tb xmin xmax ymin ymax zmin zmax...To change the bounding Cube.\n"); 
	fprintf(stderr,"\t\tCurrent bounding cube extents are :\n");
	fprintf(stderr,"\t\txmin:%g xmax:%g ymin:%g ymax:%g zmin:%g zmax:%g\n",
			xmin,xmax,ymin,ymax,zmin,zmax);
	fprintf(stderr,"\tc <circle_precision>...To change the number of lines in which a circle is drawn.\n");
	fprintf(stderr,"\t\tCurrent circle precision is %d.\n",circle_precision);
	fprintf(stderr,"\tv <ONE/FOUR>... To change the View from a choice of One or Four.\n");
	fprintf(stderr,"\t\tCurrent view is %s.\n", (view_type==ONE)?"ONE":"FOUR");
}
draw_partition()
{
        color(WHITE);
        Xdraw(window_x_size/2,0,window_x_size/2,window_y_size);
        Xdraw(0,window_y_size/2,window_x_size,window_y_size/2);
}
redraw()
{
	color(BLACK);
	clear(0,0,window_x_size,window_y_size);
	if (view_type == FOUR) draw_partition();
	readobj_n_draw(filename);
}
main(argc, argv)
	int	argc;
	char	**argv;
{
	char s[120];
	char command[80];
	int 	done=0;

	parse(argc, argv);
	initX(window_x_posn,window_y_posn,window_x_size,window_y_size); 
        if (view_type == FOUR)draw_partition();

	set_screen_transformation(xmin,xmax,ymin,ymax,zmin,zmax);

	if (readobj_n_draw(filename)==0)
		fprintf(stderr,"New Scene File.\n");
	do{
		fprintf(stderr,"----->"); gets(s);
		switch(toupper(s[0])){
		case 'N':{
			char newfilename[80];
			if (sscanf(s,"%s%s",command,newfilename)==2){
				if (strcmp(newfilename,filename)!=0){
					strcpy(filename,newfilename);
					fprintf(stderr,"New File %s.\n",filename);
					redraw();
				}
				else fprintf(stderr,"Same file.\n");
			}
			else fprintf(stderr,"Missing File name.\n");
			}
			break;
		case 'B':{
			float x1,x2,y1,y2,z1,z2;
			if ((sscanf(s,"%s%f%f%f%f%f%f",command,&x1,&x2,&y1,&y2,&z1,&z2)!=7)
				||
			    (x2 <= x1) || (y2 <= y1) || (z2 <= z1)){
					fprintf(stderr,"Insufficient or wrong input.\n");
				fprintf(stderr,"Current Bounds are :\n");
				fprintf(stderr,"\txmin : %g, xmax : %g\n",xmin,xmax);
				fprintf(stderr,"\tymin : %g, ymax : %g\n",ymin,ymax);
				fprintf(stderr,"\tzmin : %g, zmax : %g\n",zmin,zmax);
			}
			else{
				fprintf(stderr,"%f %f %f %f %f %f\n",x1,x2,y1,y2,z1,z2);
				if ((xmin != x1)||(ymin!=y1) ||(zmin!=z1)
					||(xmax!=x2)||(ymax!=y2)||(zmax!=z2)){
					xmin = x1; xmax = x2; 
					ymin = y1; ymax = y2; 
					zmin = z1; zmax = z2;
					set_screen_transformation(xmin,xmax,ymin,ymax,zmin,zmax);
					redraw();
				}
			}
			}
			break;
		case 'C':{
			int n;
			if ((sscanf(s,"%s%d",command,&n)==2) && (n > 2)){
				if (n != circle_precision)
					circle_precision = n;
			}
			else fprintf(stderr,"Wrong input for circle precision.\n");
			}
			break;
		case 'R': redraw(); break;
		case 'V':{
			char view[80];
			if (sscanf(s,"%s%s",command,view)==2){
				int i,n=strlen(view);
				for(i=0;i<n;i++)view[i]=toupper(view[i]);
				if((strcmp(view,"ONE")==0)&&(view_type!=ONE)){
					view_type = ONE;
					set_screen_transformation(xmin,xmax,ymin,ymax,zmin,zmax);
					redraw();
				}
				else if ((strcmp(view,"FOUR")==0)&&(view_type!=FOUR)){
					view_type = FOUR;
					set_screen_transformation(xmin,xmax,ymin,ymax,zmin,zmax);
					redraw();
				}
				else fprintf(stderr,"Same or Wrong View specified.\n");
			}
			else fprintf(stderr,"View type missing.\n");
			}
			break;
		case 'Q':{
			fprintf(stderr,"Want to quit ?  y/n : ");
			gets(s);
			if (toupper(s[0]) == 'Y') done = 1;
			}
			break;
		case 'E':{
			char s[80];
			sprintf(s,"vi %s\n",filename);
			system(s);redraw();
			}
			break;
		case '?':
		case 'H':
		default : help(); break;
		}
	}while(!done);
	deinitX();
}

